@@ -5,13 +5,13 @@ import redis
 from hashring import HashRing
 import functools
 
-_findhash = re.compile('.+\{(.*)\}.*', re.I)
+_findhash = re.compile('.*\{(.*)\}.*', re.I)
 
 class RedisShardAPI(object):
 
     def __init__(self,servers):
         VERSION = tuple(map(int, redis.__version__.split('.')))
-        nodes = []
+        self.nodes = []
         self.connections = {}
         if VERSION < (2,4,0):
             self.pool = redis.ConnectionPool()
@@ -23,11 +23,18 @@ class RedisShardAPI(object):
             if name in self.connections:
                 raise ValueError("server's name config must be unique")
             self.connections[name] = conn
-            nodes.append(name)
-        self.ring = HashRing(nodes)
+            self.nodes.append(name)
+        self.ring = HashRing(self.nodes)
 
-    def get_server(self, key):
+    def get_server_name(self, key):
+        g = _findhash.match(key)
+        if g != None and len(g.groups()) > 0:
+            key = g.groups()[0]
         name = self.ring.get_node(key)
+        return name
+
+    def get_server(self,key):
+        name = self.get_server_name(key)
         return self.connections[name]
 
     def __wrap(self, method, *args, **kwargs):
@@ -36,9 +43,6 @@ class RedisShardAPI(object):
             assert isinstance(key, basestring)
         except:
             raise ValueError("method '%s' requires a key param as the first argument" % method)
-        g = _findhash.match(key)
-        if g != None and len(g.groups()) > 0:
-            key = g.groups()[0]
         server = self.get_server(key)
         f = getattr(server, method)
         return f(*args, **kwargs)
@@ -50,11 +54,11 @@ class RedisShardAPI(object):
             "incr", "decr", "exists",
             "delete", "get_type", "rename",
             "expire", "ttl", "push",
-            "llen", "lrange", "ltrim",
+            "llen", "lrange", "ltrim","lpush","lpop",
             "lindex", "pop", "lset",
             "lrem", "sadd", "srem",
             "sismember", "smembers",
-            "zadd", "zrem", "zincr",
+            "zadd", "zrem", "zincr","zrank",
             "zrange", "zrevrange", "zrangebyscore","zremrangebyrank",
             "zremrangebyscore", "zcard", "zscore",
             "hget", "hset", "hdel", "hincrby", "hlen",
@@ -65,3 +69,14 @@ class RedisShardAPI(object):
         else:
             raise NotImplementedError("method '%s' cannot be sharded" % method)
 
+
+    #########################################
+    ###  some methods implement as needed ###
+    ########################################
+
+    def keys(self,key):
+        _keys = []
+        for node in self.nodes:
+            server = self.connections[node]
+            _keys.extend(server.keys(key))
+        return _keys
